package com.mortals.xhx.module.stat.web;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.ThreadPool;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.site.SitePdu;
import com.mortals.xhx.feign.site.ISiteFeign;
import com.mortals.xhx.module.access.model.AccessEntity;
import com.mortals.xhx.module.access.model.AccessQuery;
import com.mortals.xhx.module.access.model.AccessSystemEntity;
import com.mortals.xhx.module.access.service.AccessService;
import com.mortals.xhx.module.access.service.AccessStatLogService;
import com.mortals.xhx.module.pj.model.PjEvaluateQuery;
import com.mortals.xhx.module.pj.model.PjEvaluateStatEntity;
import com.mortals.xhx.module.stat.model.StatQuery;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.stat.model.StatEntity;
import com.mortals.xhx.module.stat.service.StatService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;
import static java.util.stream.Collectors.collectingAndThen;
import static java.util.stream.Collectors.toCollection;

import com.mortals.xhx.common.code.*;

/**
 * 汇总统计
 *
 * @author zxfei
 * @date 2024-07-02
 */
@RestController
@RequestMapping("stat")
@Slf4j
public class StatController extends BaseCRUDJsonBodyMappingController<StatService, StatEntity, Long> {


    @Autowired
    private ISiteFeign siteFeign;
    @Autowired
    private StatService statService;

    @Autowired
    private AccessService accessService;
    @Autowired
    private AccessStatLogService accessStatLogService;


    public StatController() {
        super.setModuleDesc("汇总统计");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        super.init(model, context);
    }


    @PostMapping(value = "/stat")
    @UnAuth
    public Rest<String> stat(@RequestBody StatQuery query) {
        Rest<String> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = 1;
        try {

            Runnable runnable = new Runnable() {
                @Override
                public void run() {
                    //天数区间分段计算
                    DateTime attendStart = DateUtil.parseDate(query.getCreateTimeStart());
                    DateTime attendEnd = DateUtil.parseDate(query.getCreateTimeEnd());
                    Long compare = DateUtil.between(attendEnd, attendStart, DateUnit.DAY);
                    StopWatch stopWatch = new StopWatch("stopwatch");
                    log.info("计算天数区间：{}", compare);

                    AccessQuery accessQuery = new AccessQuery();
              /*      accessQuery.setTagNotList(Arrays.asList(""));
                    List<AccessEntity> accessEntities = accessService.find(accessQuery);*/

                    List<AccessEntity> accessEntities  = accessService.find(accessQuery).stream().filter(item -> {
                        List<AccessSystemEntity> accessSystemList = item.getAccessSystemList();
                        //判断排队机或者评价系统是否开通
                        if(ObjectUtils.isEmpty(accessSystemList))return false;
                        List<AccessSystemEntity> collect = accessSystemList.stream().filter(f -> f.getEnabled() == YesNoEnum.YES.getValue()).collect(Collectors.toList());
                        if(ObjectUtils.isEmpty(collect)){return false;}
                        return true;
                    }).collect(Collectors.toList());

                    List<SitePdu> sitePduList = new ArrayList<>();
                    for (AccessEntity accessEntity : accessEntities) {
                        SitePdu sitePdu = new SitePdu();
                        sitePdu.setId(accessEntity.getId());
                        Rest<List<SitePdu>> sitesRest = siteFeign.getFlatSitesBySiteId(sitePdu);
                        if (YesNoEnum.YES.getValue() == sitesRest.getCode()) {
                            sitePduList.addAll(sitesRest.getData());
                        }
                    }

                    if (!ObjectUtils.isEmpty(sitePduList)) {
                        sitePduList = sitePduList.stream().collect(
                                collectingAndThen(
                                        toCollection(() -> new TreeSet<>(Comparator.comparing(SitePdu::getId))), ArrayList::new));
                        for (SitePdu site : sitePduList) {
                            for (int i = 0; i <= compare.intValue(); i++) {
                                DateTime curDate = DateUtil.offsetDay(attendStart, i);

                                StatEntity statEntity = new StatEntity();
                                statEntity.initAttrValue();
                                statEntity.setSiteId(site.getId());
                                statEntity.setSiteName(site.getSiteName());
                                statEntity.setSiteCode(site.getSiteCode());

                                statEntity.setYear(curDate.year());
                                statEntity.setMonth(curDate.month() + 1);
                                statEntity.setDay(curDate.dayOfMonth());
                                //设置年月日
                                statService.updateSiteStat(statEntity, context);

                            }
                        }
                    }

/*
                    SitePdu sitePdu = new SitePdu();
                    sitePdu.setSize(-1);
                    Rest<RespData<List<SitePdu>>> resp = siteFeign.list(sitePdu);
                    for (int i = 0; i <= compare.intValue(); i++) {
                        DateTime curDate = DateUtil.offsetDay(attendStart, i);
                        log.info("记录日期：{}", curDate.toDateStr());
                        stopWatch.start("执行本地方法");
                        if (resp.getCode() == 1) {
                            List<SitePdu> sitePduList = resp.getData().getData();
                            sitePduList.stream().forEach(site -> {
                                StatEntity statEntity = new StatEntity();
                                statEntity.initAttrValue();
                                statEntity.setSiteId(site.getId());
                                statEntity.setSiteName(site.getSiteName());
                                statEntity.setSiteCode(site.getSiteCode());

                                statEntity.setYear(curDate.year());
                                statEntity.setMonth(curDate.month() + 1);
                                statEntity.setDay(curDate.dayOfMonth());
                                //设置年月日
                                statService.updateSiteStat(statEntity, context);
                            });
                        }

                        stopWatch.stop();
                        log.info("日期：{} 完成,耗时：{}ms", curDate.toDateStr(), stopWatch.getLastTaskTimeMillis());
                    }*/
                }
            };
            ThreadPool.getInstance().execute(runnable);

            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = -1;
            this.doException(this.request, busiDesc, model, e);
            model.put("message_info", e.getMessage());
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;

    }

}